# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_jsonization_of_classes_with_descendants.py
# Do NOT edit or append.


"""Test JSON de/serialization of classes with descendants."""


# pylint: disable=missing-docstring


import unittest

import aas_core3.jsonization as aas_jsonization

import tests.common
import tests.common_jsonization


class Test_Referable(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.Referable`.
    """

    def test_round_trip_over_relationship_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_relationship_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_annotated_relationship_element(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_asset_administration_shell(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_asset_administration_shell()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_basic_event_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_basic_event_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_blob(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_blob()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_capability(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_capability()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_concept_description(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_concept_description()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_entity(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_entity()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_file(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_file()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_multi_language_property(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_multi_language_property()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_operation(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_operation()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_property(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_property()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_range(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_range()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_reference_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_reference_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_submodel()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_collection(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_collection()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_list(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_list()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.referable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Identifiable(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.Identifiable`.
    """

    def test_round_trip_over_asset_administration_shell(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_asset_administration_shell()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.identifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_concept_description(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_concept_description()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.identifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_submodel()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.identifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Qualifiable(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.Qualifiable`.
    """

    def test_round_trip_over_relationship_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_relationship_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_annotated_relationship_element(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_basic_event_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_basic_event_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_blob(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_blob()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_capability(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_capability()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_entity(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_entity()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_file(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_file()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_multi_language_property(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_multi_language_property()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_operation(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_operation()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_property(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_property()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_range(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_range()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_reference_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_reference_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_submodel()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_collection(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_collection()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_list(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_list()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.qualifiable_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_SubmodelElement(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.SubmodelElement`.
    """

    def test_round_trip_over_relationship_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_relationship_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_annotated_relationship_element(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_basic_event_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_basic_event_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_blob(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_blob()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_capability(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_capability()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_entity(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_entity()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_file(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_file()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_multi_language_property(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_multi_language_property()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_operation(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_operation()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_property(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_property()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_range(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_range()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_reference_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_reference_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_collection(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_collection()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_submodel_element_list(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_submodel_element_list()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.submodel_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_RelationshipElement(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.RelationshipElement`.
    """

    def test_round_trip_over_annotated_relationship_element(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.relationship_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_DataElement(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.DataElement`.
    """

    def test_round_trip_over_blob(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_blob()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_file(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_file()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_multi_language_property(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_multi_language_property()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_property(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_property()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_range(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_range()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )

    def test_round_trip_over_reference_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_reference_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_EventElement(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.EventElement`.
    """

    def test_round_trip_over_basic_event_element(self) -> None:
        concrete_instance = tests.common_jsonization.load_maximal_basic_event_element()
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.event_element_from_jsonable(jsonable)
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_DataSpecificationContent(unittest.TestCase):
    """
    Test round-trip JSON de/serialization over concrete descendants
    of :py:class:`aas_types.DataSpecificationContent`.
    """

    def test_round_trip_over_data_specification_iec_61360(self) -> None:
        concrete_instance = (
            tests.common_jsonization.load_maximal_data_specification_iec_61360()
        )
        jsonable = aas_jsonization.to_jsonable(concrete_instance)

        abstract_instance = aas_jsonization.data_specification_content_from_jsonable(
            jsonable
        )
        another_jsonable = aas_jsonization.to_jsonable(abstract_instance)

        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


if __name__ == "__main__":
    unittest.main()


# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_jsonization_of_classes_with_descendants.py
# Do NOT edit or append.



"""Manipulate, verify and de/serialize Asset Administration Shells."""

# Synchronize with __init__.py and changelog.rst!
__version__ = "1.1.4"
__author__ = "Marko Ristin"
__copyright__ = "2024 Contributors to aas-core3.0-python"
__license__ = "License :: OSI Approved :: MIT License"
__status__ = "Production/Stable"



"""Provide common functions shared among the modules."""

# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.


from typing import NoReturn


def assert_never(value: NoReturn) -> NoReturn:
    """
    Signal to mypy to perform an exhaustive matching.

    Please see the following page for more details:
    https://hakibenita.com/python-mypy-exhaustive-checking
    """
    assert False, f"Unhandled value: {value} ({type(value).__name__})"


# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.



"""Provide constant values of the meta-model."""

# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.

from typing import Set

import aas_core3.types as aas_types

#: Categories for :py:class:`.types.DataElement` as defined in :ref:`Constraint AASd-090 <constraint_AASd-090>`
VALID_CATEGORIES_FOR_DATA_ELEMENT: Set[str] = {"CONSTANT", "PARAMETER", "VARIABLE"}

#: Enumeration of all identifiable elements within an asset administration shell.
GENERIC_FRAGMENT_KEYS: Set[aas_types.KeyTypes] = {aas_types.KeyTypes.FRAGMENT_REFERENCE}

#: Enumeration of different key value types within a key.
GENERIC_GLOBALLY_IDENTIFIABLES: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.GLOBAL_REFERENCE
}

#: Enumeration of different key value types within a key.
AAS_IDENTIFIABLES: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.ASSET_ADMINISTRATION_SHELL,
    aas_types.KeyTypes.CONCEPT_DESCRIPTION,
    aas_types.KeyTypes.IDENTIFIABLE,
    aas_types.KeyTypes.SUBMODEL,
}

#: Enumeration of all submodel elements within an asset administration shell.
AAS_SUBMODEL_ELEMENTS_AS_KEYS: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.ANNOTATED_RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.BASIC_EVENT_ELEMENT,
    aas_types.KeyTypes.BLOB,
    aas_types.KeyTypes.CAPABILITY,
    aas_types.KeyTypes.DATA_ELEMENT,
    aas_types.KeyTypes.ENTITY,
    aas_types.KeyTypes.EVENT_ELEMENT,
    aas_types.KeyTypes.FILE,
    aas_types.KeyTypes.MULTI_LANGUAGE_PROPERTY,
    aas_types.KeyTypes.OPERATION,
    aas_types.KeyTypes.PROPERTY,
    aas_types.KeyTypes.RANGE,
    aas_types.KeyTypes.REFERENCE_ELEMENT,
    aas_types.KeyTypes.RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_COLLECTION,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_LIST,
}

#: Enumeration of different fragment key value types within a key.
AAS_REFERABLE_NON_IDENTIFIABLES: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.ANNOTATED_RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.BASIC_EVENT_ELEMENT,
    aas_types.KeyTypes.BLOB,
    aas_types.KeyTypes.CAPABILITY,
    aas_types.KeyTypes.DATA_ELEMENT,
    aas_types.KeyTypes.ENTITY,
    aas_types.KeyTypes.EVENT_ELEMENT,
    aas_types.KeyTypes.FILE,
    aas_types.KeyTypes.MULTI_LANGUAGE_PROPERTY,
    aas_types.KeyTypes.OPERATION,
    aas_types.KeyTypes.PROPERTY,
    aas_types.KeyTypes.RANGE,
    aas_types.KeyTypes.REFERENCE_ELEMENT,
    aas_types.KeyTypes.RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_COLLECTION,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_LIST,
}

#: Enumeration of referables. We need this to check that model references refer to a Referable. For example, the observed attribute of the Basic Event Element object must be a model reference to a Referable.
AAS_REFERABLES: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.ASSET_ADMINISTRATION_SHELL,
    aas_types.KeyTypes.CONCEPT_DESCRIPTION,
    aas_types.KeyTypes.IDENTIFIABLE,
    aas_types.KeyTypes.SUBMODEL,
    aas_types.KeyTypes.ANNOTATED_RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.BASIC_EVENT_ELEMENT,
    aas_types.KeyTypes.BLOB,
    aas_types.KeyTypes.CAPABILITY,
    aas_types.KeyTypes.DATA_ELEMENT,
    aas_types.KeyTypes.ENTITY,
    aas_types.KeyTypes.EVENT_ELEMENT,
    aas_types.KeyTypes.FILE,
    aas_types.KeyTypes.MULTI_LANGUAGE_PROPERTY,
    aas_types.KeyTypes.OPERATION,
    aas_types.KeyTypes.PROPERTY,
    aas_types.KeyTypes.RANGE,
    aas_types.KeyTypes.REFERENCE_ELEMENT,
    aas_types.KeyTypes.REFERABLE,
    aas_types.KeyTypes.RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_COLLECTION,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_LIST,
}

#: Enumeration of all referable elements within an asset administration shell
GLOBALLY_IDENTIFIABLES: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.GLOBAL_REFERENCE,
    aas_types.KeyTypes.ASSET_ADMINISTRATION_SHELL,
    aas_types.KeyTypes.CONCEPT_DESCRIPTION,
    aas_types.KeyTypes.IDENTIFIABLE,
    aas_types.KeyTypes.SUBMODEL,
}

#: Enumeration of different key value types within a key.
FRAGMENT_KEYS: Set[aas_types.KeyTypes] = {
    aas_types.KeyTypes.ANNOTATED_RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.BASIC_EVENT_ELEMENT,
    aas_types.KeyTypes.BLOB,
    aas_types.KeyTypes.CAPABILITY,
    aas_types.KeyTypes.DATA_ELEMENT,
    aas_types.KeyTypes.ENTITY,
    aas_types.KeyTypes.EVENT_ELEMENT,
    aas_types.KeyTypes.FILE,
    aas_types.KeyTypes.FRAGMENT_REFERENCE,
    aas_types.KeyTypes.MULTI_LANGUAGE_PROPERTY,
    aas_types.KeyTypes.OPERATION,
    aas_types.KeyTypes.PROPERTY,
    aas_types.KeyTypes.RANGE,
    aas_types.KeyTypes.REFERENCE_ELEMENT,
    aas_types.KeyTypes.RELATIONSHIP_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_COLLECTION,
    aas_types.KeyTypes.SUBMODEL_ELEMENT_LIST,
}

#: IEC 61360 data types for concept descriptions categorized with PROPERTY or VALUE.
DATA_TYPE_IEC_61360_FOR_PROPERTY_OR_VALUE: Set[aas_types.DataTypeIEC61360] = {
    aas_types.DataTypeIEC61360.DATE,
    aas_types.DataTypeIEC61360.STRING,
    aas_types.DataTypeIEC61360.STRING_TRANSLATABLE,
    aas_types.DataTypeIEC61360.INTEGER_MEASURE,
    aas_types.DataTypeIEC61360.INTEGER_COUNT,
    aas_types.DataTypeIEC61360.INTEGER_CURRENCY,
    aas_types.DataTypeIEC61360.REAL_MEASURE,
    aas_types.DataTypeIEC61360.REAL_COUNT,
    aas_types.DataTypeIEC61360.REAL_CURRENCY,
    aas_types.DataTypeIEC61360.BOOLEAN,
    aas_types.DataTypeIEC61360.RATIONAL,
    aas_types.DataTypeIEC61360.RATIONAL_MEASURE,
    aas_types.DataTypeIEC61360.TIME,
    aas_types.DataTypeIEC61360.TIMESTAMP,
}

#: IEC 61360 data types for concept descriptions categorized with REFERENCE.
DATA_TYPE_IEC_61360_FOR_REFERENCE: Set[aas_types.DataTypeIEC61360] = {
    aas_types.DataTypeIEC61360.STRING,
    aas_types.DataTypeIEC61360.IRI,
    aas_types.DataTypeIEC61360.IRDI,
}

#: IEC 61360 data types for concept descriptions categorized with DOCUMENT.
DATA_TYPE_IEC_61360_FOR_DOCUMENT: Set[aas_types.DataTypeIEC61360] = {
    aas_types.DataTypeIEC61360.FILE,
    aas_types.DataTypeIEC61360.BLOB,
    aas_types.DataTypeIEC61360.HTML,
}

#: These data types imply that the unit is defined in the data specification.
IEC_61360_DATA_TYPES_WITH_UNIT: Set[aas_types.DataTypeIEC61360] = {
    aas_types.DataTypeIEC61360.INTEGER_MEASURE,
    aas_types.DataTypeIEC61360.REAL_MEASURE,
    aas_types.DataTypeIEC61360.RATIONAL_MEASURE,
    aas_types.DataTypeIEC61360.INTEGER_CURRENCY,
    aas_types.DataTypeIEC61360.REAL_CURRENCY,
}

# This code has been automatically generated by aas-core-codegen.
# Do NOT edit or append.



"""Test the edge cases with the ElementTree parser."""

# pylint: disable=missing-docstring
import io
import unittest
import xml.etree.ElementTree
from typing import Tuple, Iterator

import aas_core3.xmlization as aas_xmlization


class TestTextAttachedToAnEndElement(unittest.TestCase):
    """
    Test the edge case when the ElementTree attaches the text to the end element.

    The quirky part is that the text is *sometimes* attached to the start element
    and *other times* to the end element. We could not figure out which situation
    occurs when.

    Please see also the note in the Python documentation:
    https://docs.python.org/3/library/xml.etree.elementtree.html#xml.etree.ElementTree.XMLPullParser.read_events
    """

    def test_on_a_large_file(self) -> None:
        # Please see:
        # https://github.com/aas-core-works/aas-core3.0-python/issues/17
        # and
        # https://github.com/aas-core-works/aas-core-codegen/pull/443
        text = """\
<environment xmlns="https://admin-shell.io/aas/3/0">
  <assetAdministrationShells>
    <assetAdministrationShell>
      <idShort>NameplateTemplateTestShell</idShort>
      <id>AssetAdministrationShell</id>
      <assetInformation>
        <assetKind>NotApplicable</assetKind>
      </assetInformation>
      <submodels>
        <reference>
          <type>ModelReference</type>
          <keys>
            <key>
              <type>Submodel</type>
              <value>www.example.com/ids/sm/1225_9020_5022_1974</value>
            </key>
          </keys>
        </reference>
      </submodels>
    </assetAdministrationShell>
  </assetAdministrationShells>
  <submodels>
    <submodel>
      <idShort>Nameplate</idShort>
      <description>
        <langStringTextType>
          <language>en</language>
          <text>Contains the nameplate information attached to the product</text>
        </langStringTextType>
      </description>
      <id>www.example.com/ids/sm/1225_9020_5022_1974</id>
      <kind>Template</kind>
      <semanticId>
        <type>ExternalReference</type>
        <keys>
          <key>
            <type>ConceptDescription</type>
            <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate</value>
          </key>
        </keys>
      </semanticId>
      <submodelElements>
        <file>
          <idShort>CompanyLogo</idShort>
          <semanticId>
            <type>ExternalReference</type>
            <keys>
              <key>
                <type>GlobalReference</type>
                <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/CompanyLogo</value>
              </key>
            </keys>
          </semanticId>
          <qualifiers>
            <qualifier>
              <type>Multiplicity</type>
              <valueType>xs:string</valueType>
              <value>ZeroToOne</value>
            </qualifier>
          </qualifiers>
          <value></value>
          <contentType></contentType>
        </file>
        <submodelElementList>
          <idShort>Markings</idShort>
          <description>
            <langStringTextType>
              <language>en</language>
              <text>Note: CE marking is declared as mandatory according to EU Machine Directive 2006/42/EC.</text>
            </langStringTextType>
          </description>
          <typeValueListElement>SubmodelElementCollection</typeValueListElement>
          <orderRelevant>false</orderRelevant>
          <semanticId>
            <type>ExternalReference</type>
            <keys>
              <key>
                <type>GlobalReference</type>
                <value>0173-1#01-AGZ673#001</value>
              </key>
            </keys>
          </semanticId>
          <semanticIdListElement>
              <type>ExternalReference</type>
              <keys>
                  <key>
                      <type>GlobalReference</type>
                      <value>0173-1#01-AHD206#001</value>
                  </key>
              </keys>
          </semanticIdListElement>
          <qualifiers>
            <qualifier>
              <type>Multiplicity</type>
              <valueType>xs:string</valueType>
              <value>ZeroToOne</value>
            </qualifier>
          </qualifiers>
          <value>
            <submodelElementCollection>
              <idShort>Marking</idShort>
              <description>
                <langStringTextType>
                  <language>en</language>
                  <text>Note: see also [IRDI] 0112/2///61987#ABH515#003 Certificate or approval Note: CE marking is declared as mandatory according to the Blue Guide of the EU-Commission </text>
                </langStringTextType>
              </description>
              <semanticId>
                <type>ExternalReference</type>
                <keys>
                  <key>
                    <type>GlobalReference</type>
                    <value>0173-1#01-AHD206#001</value>
                  </key>
                </keys>
              </semanticId>
              <value>
                <property>
                  <idShort>ExpiryDate</idShort>
                  <description>
                    <langStringTextType>
                      <language>en</language>
                      <text>Note: see also ([IRDI] 0173-1#02-AAO997#001 Validity date Note: format by lexical representation: CCYY-MM-DD Note: to be specified to the day </text>
                    </langStringTextType>
                  </description>
                  <semanticId>
                    <type>ExternalReference</type>
                    <keys>
                      <key>
                        <type>GlobalReference</type>
                        <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExpiryDate</value>
                      </key>
                    </keys>
                  </semanticId>
                  <qualifiers>
                    <qualifier>
                      <type>Multiplicity</type>
                      <valueType>xs:string</valueType>
                      <value>ZeroToOne</value>
                    </qualifier>
                  </qualifiers>
                  <valueType>xs:string</valueType>
                  <value>2022-01-01</value>
                </property>
                <submodelElementList>
                  <idShort>ExplosionSafeties</idShort>
                  <typeValueListElement>SubmodelElementCollection</typeValueListElement>
                  <orderRelevant>false</orderRelevant>
                  <semanticId>
                    <type>ExternalReference</type>
                    <keys>
                      <key>
                        <type>GlobalReference</type>
                        <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties</value>
                      </key>
                    </keys>
                  </semanticId>
                  <semanticIdListElement>
                      <type>ExternalReference</type>
                      <keys>
                          <key>
                              <type>GlobalReference</type>
                              <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety</value>
                          </key>
                      </keys>
                  </semanticIdListElement>
                  <qualifiers>
                    <qualifier>
                      <type>Multiplicity</type>
                      <valueType>xs:string</valueType>
                      <value>ZeroToOne</value>
                    </qualifier>
                  </qualifiers>
                  <value>
                    <submodelElementCollection>
                      <idShort>ExplosionSafety</idShort>
                      <semanticId>
                        <type>ExternalReference</type>
                        <keys>
                          <key>
                            <type>GlobalReference</type>
                            <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety</value>
                          </key>
                        </keys>
                      </semanticId>
                      <qualifiers>
                        <qualifier>
                          <type>Multiplicity</type>
                          <valueType>xs:string</valueType>
                          <value>OneToMany</value>
                        </qualifier>
                      </qualifiers>
                      <value>
                        <property>
                          <idShort>DesignationOfCertificateOrApproval</idShort>
                          <description>
                            <langStringTextType>
                              <language>en</language>
                              <text>Note: Approval identifier, reference to the certificate number, to be entered without spaces </text>
                            </langStringTextType>
                          </description>
                          <semanticId>
                            <type>ExternalReference</type>
                            <keys>
                              <key>
                                <type>GlobalReference</type>
                                <value>0112/2///61987#ABH783#001</value>
                              </key>
                            </keys>
                          </semanticId>
                          <qualifiers>
                            <qualifier>
                              <type>Multiplicity</type>
                              <valueType>xs:string</valueType>
                              <value>ZeroToOne</value>
                            </qualifier>
                          </qualifiers>
                          <valueType>xs:string</valueType>
                          <value>KEMA99IECEX1105/128</value>
                        </property>
                        <multiLanguageProperty>
                          <idShort>TypeOfApproval</idShort>
                          <description>
                            <langStringTextType>
                              <language>en</language>
                              <text>Note: see also [IRDI] 0112/2///61987#ABA231#008 type of hazardous area approval Note: name of the approval system, e.g. ATEX, IECEX, NEC, EAC, CCC, CEC Note: only values from the enumeration should be used as stated. For additional systems further values can be used. Note: Recommendation: property declaration as MLP is required by its semantic definition. As the property value is language independent, users are recommended to provide maximal 1 string in any language of the user’s choice.</text>
                            </langStringTextType>
                          </description>
                          <semanticId>
                            <type>ExternalReference</type>
                            <keys>
                              <key>
                                <type>GlobalReference</type>
                                <value>0173-1#02-AAM812#003</value>
                              </key>
                            </keys>
                          </semanticId>
                          <qualifiers>
                            <qualifier>
                              <type>Multiplicity</type>
                              <valueType>xs:string</valueType>
                              <value>ZeroToOne</value>
                            </qualifier>
                          </qualifiers>
                          <value>
                            <langStringTextType>
                              <language>en</language>
                              <text>ATEX</text>
                            </langStringTextType>
                          </value>
                        </multiLanguageProperty>
                        <property>
                          <idShort>IncompleteDevice</idShort>
                          <semanticId>
                            <type>ExternalReference</type>
                            <keys>
                              <key>
                                <type>GlobalReference</type>
                                <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety/IncompleteDevice</value>
                              </key>
                            </keys>
                          </semanticId>
                          <qualifiers>
                            <qualifier>
                              <type>Multiplicity</type>
                              <valueType>xs:string</valueType>
                              <value>ZeroToOne</value>
                            </qualifier>
                          </qualifiers>
                          <valueType>xs:string</valueType>
                          <value>U</value>
                        </property>
                        <submodelElementCollection>
                          <idShort>AmbientConditions</idShort>
                          <description>
                            <langStringTextType>
                              <language>en</language>
                              <text>Note: If the device is mounted in the process boundary, ambient and process conditions are provided separately. </text>
                            </langStringTextType>
                          </description>
                          <semanticId>
                            <type>ExternalReference</type>
                            <keys>
                              <key>
                                <type>GlobalReference</type>
                                <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety/AmbientConditions</value>
                              </key>
                            </keys>
                          </semanticId>
                          <qualifiers>
                            <qualifier>
                              <type>Multiplicity</type>
                              <valueType>xs:string</valueType>
                              <value>ZeroToOne</value>
                            </qualifier>
                          </qualifiers>
                          <value>
                            <property>
                              <idShort>DeviceCategory</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA467#002 equipment/device category Note: editorial definiton: Category of device in accordance with directive 2014/34/EU Note: Equipment category according to the ATEX system. According to the current nameplate, also the combination “GD” is permitted Note: The combination “GD” is no longer accepted and was changed in the standards. Currently the marking for “G” and “D” must be provided in a separate marking string. Older devices may still exist with the marking “GD”. </text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAK297#004</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>2G</value>
                            </property>
                            <multiLanguageProperty>
                              <idShort>EquipmentProtectionLevel</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA464#005 equipment protection level Note: editorial definition: Level of protection assigned to equipment based on its likelihood of becoming a source of ignition Note: Equipment protection level according to the IEC standards. According to the current nameplate, also the combination “GD” is permitted Note: The combination “GD” is no longer accepted and was changed in the standards. Currently the marking for “G” and “D” must be provided in a separate marking string. Older devices may still exist with the marking “GD”. Note: Recommendation: property declaration as MLP is required by its semantic definition. As the property value is language independent, users are recommended to provide maximal 1 string in any language of the user’s choice.</text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAM668#001</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <value>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Gb</text>
                                </langStringTextType>
                              </value>
                            </multiLanguageProperty>
                            <property>
                              <idShort>RegionalSpecificMarking</idShort>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety/RegionalSpecificMarking</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>Class I, Division 2</value>
                            </property>
                            <property>
                              <idShort>TypeOfProtection</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA589#002 type of protection (Ex) Note: Symbol(s) for the Type of protection. Several types of protection are separated by a semicolon “;” </text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAQ325#003</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>db</value>
                            </property>
                            <property>
                              <idShort>ExplosionGroup</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA961#007 permitted gas group/explosion group Note: Equipment grouping according to IEC 60079-0 is meant by this property Note: Symbol(s) for the gas group (IIA…IIC) or dust group (IIIA…IIIC) </text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAT372#001</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>IIC</value>
                            </property>
                            <property>
                              <idShort>MinimumAmbientTemperature</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA621#007 minimum ambient temperature Note: editorial defnition: lower limit of the temperature range of the environment in which the component, the pipework or the system can be operated Note: Rated minimum ambient temperature Note: Positive temperatures are listed without “+” sign. If several temperatures ranges are marked, only the most general range shall be indicated in the template, which is consistent with the specified temperature class or maximum surface temperature. Other temperature ranges and temperature classes/maximum surface temperatures may be listed in the instructions.</text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAZ952#001</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>-40</value>
                            </property>
                            <property>
                              <idShort>MaxAmbientTemperature</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA623#007 maximum ambient temperature Note: editorial definition: upper limit of the temperature range of the environment in which the component, the pipework or the system can be operated Note: Rated maximum ambient temperature Note: Positive temperatures are listed without “+” sign. If several temperatures ranges are marked, only the most general range shall be indicated in the template, which is consistent with the specified temperature class or maximum surface temperature. Other temperature ranges and temperature classes/maximum surface temperatures may be listed in the instructions.</text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-BAA039#010</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>120</value>
                            </property>
                            <property>
                              <idShort>MaxSurfaceTemperatureForDustProof</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABB159#004 maximum surface temperature for dust-proof Note: Maximum surface temperature of the device (dust layer ≤ 5 mm) for specified maximum ambient and maximum process temperature, relevant for Group III only Note: Positive temperatures are listed without “+” sign. If several temperatures ranges are marked, only the most general range shall be indicated in the template, which is consistent with the specified temperature class or maximum surface temperature. Other temperature ranges and temperature classes/maximum surface temperatures may be listed in the instructions.</text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAM666#005</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>100</value>
                            </property>
                            <property>
                              <idShort>TemperatureClass</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA593#002 temperature class Note: editorial definition: classification system of electrical apparatus, based on its maximum surface temperature, intended for use in an explosive atmospheres with flammable gas, vapour or mist. Note: Temperature class for specified maximum ambient and maximum process temperature, relevant for Group II only (Further combinations may be provided in the instruction manual). </text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAO371#004</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>T4</value>
                            </property>
                          </value>
                        </submodelElementCollection>
                        <submodelElementCollection>
                          <idShort>ProcessConditions</idShort>
                          <description>
                            <langStringTextType>
                              <language>en</language>
                              <text>Note: If the device is mounted in the process boundary, ambient and process conditions are provided separately. </text>
                            </langStringTextType>
                          </description>
                          <semanticId>
                            <type>ExternalReference</type>
                            <keys>
                              <key>
                                <type>GlobalReference</type>
                                <value>https://admin-shell.io/zvei/nameplate/2/0/Nameplate/Markings/Marking/ExplosionSafeties/ExplosionSafety/ProcessConditions</value>
                              </key>
                            </keys>
                          </semanticId>
                          <qualifiers>
                            <qualifier>
                              <type>Multiplicity</type>
                              <valueType>xs:string</valueType>
                              <value>ZeroToOne</value>
                            </qualifier>
                          </qualifiers>
                          <value>
                            <property>
                              <idShort>DeviceCategory</idShort>
                              <description>
                                <langStringTextType>
                                  <language>en</language>
                                  <text>Note: see also [IRDI] 0112/2///61987#ABA467#002 equipment/device category Note: editorial defnition: Category of device in accordance with directive 2014/34/EU Note: Equipment category according to the ATEX system. </text>
                                </langStringTextType>
                              </description>
                              <semanticId>
                                <type>ExternalReference</type>
                                <keys>
                                  <key>
                                    <type>GlobalReference</type>
                                    <value>0173-1#02-AAK297#004</value>
                                  </key>
                                </keys>
                              </semanticId>
                              <qualifiers>
                                <qualifier>
                                  <type>Multiplicity</type>
                                  <valueType>xs:string</valueType>
                                  <value>ZeroToOne</value>
                                </qualifier>
                              </qualifiers>
                              <valueType>xs:string</valueType>
                              <value>1G</value>
                            </property>
                          </value>
                        </submodelElementCollection>
                      </value>
                    </submodelElementCollection>
                  </value>
                </submodelElementList>
              </value>
            </submodelElementCollection>
          </value>
        </submodelElementList>
      </submodelElements>
    </submodel>
  </submodels>
  <conceptDescriptions>
  </conceptDescriptions>
</environment>
"""

        # NOTE (mristin, 2024-02-14):
        # This statement raised the exception before the fix. Strangely enough,
        # when the indention of the XML text changed, there was no error. This might
        # have something to do with the buffer size, but we are not sure.
        #
        # For example, if you change the indention of the XML, there would be no error.
        _ = aas_xmlization.environment_from_str(text)


class TestOnXSIAttributes(unittest.TestCase):
    """
    Test how we can deal gracefully with common ``xsi:`` attributes.

    The specification does not allow any XML attributes, but many applications simply
    add ``xsi`` attributes by default.

    See:
    https://github.com/aas-core-works/aas-core3.0-python/issues/44
    """

    def test_pipeline_approach(self) -> None:
        text = """\
<?xml version="1.0" encoding="UTF-8"?>
<environment
    xmlns="https://admin-shell.io/aas/3/0"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="https://admin-shell.io/aas/3/0/AAS.xsd">
</environment>
"""

        iterator = xml.etree.ElementTree.iterparse(
            io.StringIO(text), events=("start", "end")
        )

        def with_attributes_removed(
            an_iterator: Iterator[Tuple[str, xml.etree.ElementTree.Element]],
        ) -> Iterator[Tuple[str, xml.etree.ElementTree.Element]]:
            """
            Map the :paramref:`iterator` such that all attributes are removed.

            :param an_iterator: to be mapped
            :yield: event and element without attributes from :paramref:`iterator`
            """
            for event, element in an_iterator:
                element.attrib.clear()

                yield event, element

        environment = aas_xmlization.environment_from_iterparse(
            iterator=with_attributes_removed(iterator)
        )

        # NOTE (mristin):
        # The attributes are lost in the subsequent serialization.
        serialization = aas_xmlization.to_str(environment)

        self.assertEqual(
            '<environment xmlns="https://admin-shell.io/aas/3/0"/>',
            serialization,
        )


if __name__ == "__main__":
    unittest.main()



# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_jsonization_of_concrete_classes_outside_container.py
# Do NOT edit or append.


"""Test de/serialization from JSON of concrete classes outside a container."""


# pylint: disable=missing-docstring


import unittest

import aas_core3.jsonization as aas_jsonization

import tests.common
import tests.common_jsonization


class Test_Extension(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_extension()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.extension_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_AdministrativeInformation(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_administrative_information()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.administrative_information_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Qualifier(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_qualifier()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.qualifier_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_AssetAdministrationShell(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_administration_shell()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.asset_administration_shell_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_AssetInformation(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_asset_information()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.asset_information_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Resource(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_resource()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.resource_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_SpecificAssetID(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_specific_asset_id()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.specific_asset_id_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Submodel(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.submodel_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_RelationshipElement(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_relationship_element()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.relationship_element_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_SubmodelElementList(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_list()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.submodel_element_list_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_SubmodelElementCollection(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_submodel_element_collection()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.submodel_element_collection_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Property(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_property()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.property_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_MultiLanguageProperty(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_multi_language_property()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.multi_language_property_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Range(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_range()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.range_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_ReferenceElement(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference_element()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.reference_element_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Blob(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_blob()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.blob_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_File(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_file()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.file_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_AnnotatedRelationshipElement(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_annotated_relationship_element()
        )

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.annotated_relationship_element_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Entity(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_entity()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.entity_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_BasicEventElement(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_basic_event_element()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.basic_event_element_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Operation(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.operation_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_OperationVariable(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_operation_variable()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.operation_variable_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Capability(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_capability()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.capability_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_ConceptDescription(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_concept_description()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.concept_description_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Reference(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_reference()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.reference_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_Key(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_key()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.key_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LangStringNameType(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_name_type()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.lang_string_name_type_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LangStringTextType(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_lang_string_text_type()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.lang_string_text_type_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_EmbeddedDataSpecification(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_embedded_data_specification()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.embedded_data_specification_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LevelType(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_level_type()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.level_type_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_ValueReferencePair(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_reference_pair()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.value_reference_pair_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_ValueList(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_value_list()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.value_list_from_jsonable(jsonable)

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LangStringPreferredNameTypeIEC61360(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_preferred_name_type_iec_61360()
        )

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = (
            aas_jsonization.lang_string_preferred_name_type_iec_61360_from_jsonable(
                jsonable
            )
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LangStringShortNameTypeIEC61360(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_short_name_type_iec_61360()
        )

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = (
            aas_jsonization.lang_string_short_name_type_iec_61360_from_jsonable(
                jsonable
            )
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_LangStringDefinitionTypeIEC61360(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = (
            tests.common_jsonization.load_maximal_lang_string_definition_type_iec_61360()
        )

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = (
            aas_jsonization.lang_string_definition_type_iec_61360_from_jsonable(
                jsonable
            )
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


class Test_DataSpecificationIEC61360(unittest.TestCase):
    def test_round_trip(self) -> None:
        instance = tests.common_jsonization.load_maximal_data_specification_iec_61360()

        jsonable = aas_jsonization.to_jsonable(instance)

        another_instance = aas_jsonization.data_specification_iec_61360_from_jsonable(
            jsonable
        )

        another_jsonable = aas_jsonization.to_jsonable(another_instance)

        # Check the round-trip
        self.assertListEqual(
            [],
            list(
                map(
                    str,
                    tests.common_jsonization.check_equal(jsonable, another_jsonable),
                )
            ),
        )


if __name__ == "__main__":
    unittest.main()


# This code has been automatically generated by:
# dev_scripts/test_codegen/generate_test_for_jsonization_of_concrete_classes_outside_container.py
# Do NOT edit or append.
